@@ -68,7 +68,19 @@ def parse_config(config_string):
     inputs, outputs, labels = {}, {}, defaultdict(dict)
     for tag in xml_tree.iter():
         if _is_output_tag(tag):
-            outputs[tag.attrib['name']] = {'type': tag.tag, 'to_name': tag.attrib['toName'].split(',')}
+            tag_info = {'type': tag.tag, 'to_name': tag.attrib['toName'].split(',')}
+            # Grab conditionals if any
+            conditionals = {}
+            if tag.attrib.get('perRegion') == 'true':
+                if tag.attrib.get('whenTagName'):
+                    conditionals = {'type': 'tag', 'name': tag.attrib['whenTagName']}
+                elif tag.attrib.get('whenLabelValue'):
+                    conditionals = {'type': 'label', 'name': tag.attrib['whenLabelValue']}
+                elif tag.attrib.get('whenChoiceValue'):
+                    conditionals = {'type': 'choice', 'name': tag.attrib['whenChoiceValue']}
+            if conditionals:
+                tag_info['conditionals'] = conditionals
+            outputs[tag.attrib['name']] = tag_info
         elif _is_input_tag(tag):
             inputs[tag.attrib['name']] = {'type': tag.tag, 'value': tag.attrib['value'].lstrip('$')}
         if tag.tag not in _LABEL_TAGS:
@@ -87,12 +99,11 @@ def parse_config(config_string):
         for input_tag_name in tag_info['to_name']:
             if input_tag_name not in inputs:
                 raise KeyError('to_name={input_tag_name} is specified for output tag name={output_tag}, '
-                               'but we can\'t find it among input tags'
+                               "but we can't find it among input tags"
                                .format(input_tag_name=input_tag_name, output_tag=output_tag))
             tag_info['inputs'].append(inputs[input_tag_name])
         tag_info['labels'] = list(labels[output_tag])
         tag_info['labels_attrs'] = labels[output_tag]
-    logger.debug('Parsed config:\n' + json.dumps(outputs, indent=2))
     return outputs
 
 
@@ -341,3 +352,29 @@ def get_sample_task(label_config, secure_mode=False):
     if predefined_task is not None:
         generated_task.update(predefined_task)
     return generated_task, annotations, predictions
+
+
+def config_essential_data_has_changed(new_config_str, old_config_str):
+    """ Detect essential changes of the labeling config
+    """
+    new_config = parse_config(new_config_str)
+    old_config = parse_config(old_config_str)
+
+    for tag, new_info in new_config.items():
+        if tag not in old_config:
+            return True
+        old_info = old_config[tag]
+        if new_info['type'] != old_info['type']:
+            return True
+        if new_info['inputs'] != old_info['inputs']:
+            return True
+        if not set(old_info['labels']).issubset(new_info['labels']):
+            return True
+
+
+def replace_task_data_undefined_with_config_field(data, project):
+    # assign undefined key name from data to the first key from config, e.g. for txt loading
+    if settings.DATA_UNDEFINED_NAME in data and project.data_types.keys():
+        key = list(project.data_types.keys())[0]
+        data[key] = data[settings.DATA_UNDEFINED_NAME]
+        del data[settings.DATA_UNDEFINED_NAME]
