@@ -418,11 +418,12 @@ def sample_query(q, sample_size):
 def get_project(obj):
     from projects.models import Project, ProjectSummary
     from tasks.models import Task, Annotation, AnnotationDraft
+    from io_storages.base_models import ImportStorage
     from data_manager.models import View
 
     if isinstance(obj, Project):
         return obj
-    elif isinstance(obj, (Task, ProjectSummary, View)):
+    elif isinstance(obj, (Task, ProjectSummary, View, ImportStorage)):
         return obj.project
     elif isinstance(obj, (Annotation, AnnotationDraft)):
         return obj.task.project
@@ -617,14 +618,16 @@ def collect_versions(force=False):
 
     # label studio frontend
     try:
-        lsf = json.load(open(os.path.join(settings.EDITOR_ROOT, 'version.json')))
+        with open(os.path.join(settings.EDITOR_ROOT, 'version.json')) as f:
+            lsf = json.load(f)
         result['label-studio-frontend'] = lsf
     except:
         pass
 
     # data manager
     try:
-        dm = json.load(open(os.path.join(settings.DM_ROOT, 'version.json')))
+        with open(os.path.join(settings.DM_ROOT, 'version.json')) as f:
+            dm = json.load(f)
         result['dm2'] = dm
     except:
         pass
