@@ -57,6 +57,15 @@ def get_all_columns(project):
         task_data_children.append(column['id'])
         i += 1
 
+    # --- Data root ---
+    data_root = {
+        'id': 'data',
+        'title': "data",
+        'type': "List",
+        'target': 'tasks',
+        'children': task_data_children
+    }
+
     result['columns'] += [
         # --- Tasks ---
         {
@@ -179,15 +188,11 @@ def get_all_columns(project):
                 'explore': True,
                 'labeling': False
             }
-        },
-        {
-            'id': 'data',
-            'title': "data",
-            'type': "List",
-            'target': 'tasks',
-            'children': task_data_children
         }
     ]
+
+    result['columns'].append(data_root)
+
     return result
 
 
@@ -213,3 +218,16 @@ def get_prepared_queryset(request, project):
 
     queryset = Task.prepared.all(prepare_params=prepare_params)
     return queryset
+
+
+def evaluate_predictions(tasks):
+    """ Call ML backend for prediction evaluation of the task queryset
+    """
+    if not tasks:
+        return
+
+    project = tasks[0].project
+
+    for ml_backend in project.ml_backends.all():
+        # tasks = tasks.filter(~Q(predictions__model_version=ml_backend.model_version))
+        ml_backend.predict_many_tasks(tasks)
