@@ -6,6 +6,7 @@ import re
 from django.db import models
 from django.db.models import Aggregate, Count, Exists, OuterRef, Subquery, Avg, Q, F, Value
 from django.contrib.postgres.aggregates import ArrayAgg
+from django.db.models.fields.json import KeyTransform
 from django.db.models.functions import Coalesce
 from django.conf import settings
 
@@ -68,6 +69,7 @@ def get_fields_for_annotation(prepare_params):
     # we don't need to annotate regular model fields, so we skip them
     skipped_fields = [field.attname for field in Task._meta.fields]
     skipped_fields.append("id")
+    skipped_fields.append("file_upload")
     result = [f for f in result if f not in skipped_fields]
     result = [f for f in result if not f.startswith("data.")]
 
@@ -77,22 +79,34 @@ def get_fields_for_annotation(prepare_params):
 def apply_ordering(queryset, ordering):
     if ordering:
         field_name = ordering[0].replace("tasks:", "")
+        ascending = False if field_name[0] == '-' else True  # detect direction
+        field_name = field_name[1:] if field_name[0] == '-' else field_name  # remove direction
+
         if "data." in field_name:
             field_name = field_name.replace(".", "__", 1)
             only_undefined_field = queryset.exists() and queryset.first().project.only_undefined_field
             if only_undefined_field:
                 field_name = re.sub('data__\w+', f'data__{settings.DATA_UNDEFINED_NAME}', field_name)
 
-        ascending = False if field_name[0] == '-' else True  # detect direction
-        field_name = field_name[1:] if field_name[0] == '-' else field_name  # remove direction
+            # annotate task with data field for float/int/bool ordering support
+            json_field = field_name.replace('data__', '')
+            queryset = queryset.annotate(ordering_field=KeyTransform(json_field, 'data'))
+            f = F('ordering_field').asc(nulls_last=True) if ascending else F('ordering_field').desc(nulls_last=True)
+
+        else:
+            f = F(field_name).asc(nulls_last=True) if ascending else F(field_name).desc(nulls_last=True)
 
-        f = F(field_name).asc(nulls_last=True) if ascending else F(field_name).desc(nulls_last=True)
         queryset = queryset.order_by(f)
     else:
         queryset = queryset.order_by("id")
 
     return queryset
 
+def cast_value(_filter):
+    if _filter.type == 'Number':
+        _filter.value = float(_filter.value)
+    elif _filter.type == 'Boolean':
+        _filter.value = bool(_filter.value)
 
 def apply_filters(queryset, filters):
     if not filters:
@@ -115,6 +129,7 @@ def apply_filters(queryset, filters):
         # django orm loop expression attached to column name
         field_name = preprocess_field_name(_filter.filter, _filter.operator, only_undefined_field)
 
+        # in
         if _filter.operator == "in":
             filter_expression.add(
                 Q(
@@ -125,6 +140,8 @@ def apply_filters(queryset, filters):
                 ),
                 conjunction,
             )
+
+        # not in
         elif _filter.operator == "not_in":
             filter_expression.add(
                 ~Q(
@@ -135,15 +152,24 @@ def apply_filters(queryset, filters):
                 ),
                 conjunction,
             )
+
+        # empty
         elif _filter.operator == 'empty':
             if _filter.value == 'True':
                 filter_expression.add(Q(**{field_name: True}), conjunction)
             else:
                 filter_expression.add(~Q(**{field_name: True}), conjunction)
+
+        # starting from not_
         elif _filter.operator.startswith("not_"):
+            cast_value(_filter)
             filter_expression.add(~Q(**{field_name: _filter.value}), conjunction)
+
+        # all others
         else:
+            cast_value(_filter)
             filter_expression.add(Q(**{field_name: _filter.value}), conjunction)
+    
     logger.debug(f'Apply filter: {filter_expression}')
     queryset = queryset.filter(filter_expression)
     return queryset
@@ -232,7 +258,7 @@ def dummy(queryset):
     return queryset
 
 
-annotations_map = {
+settings.DATA_MANAGER_ANNOTATIONS_MAP = {
     "completed_at": annotate_completed_at,
     "cancelled_annotations": annotate_cancelled_annotations,
     "annotations_results": annotate_annotations_results,
@@ -244,9 +270,18 @@ annotations_map = {
 }
 
 
+def get_annotations_map():
+    return settings.DATA_MANAGER_ANNOTATIONS_MAP
+
+
+def update_annotation_map(obj):
+    settings.DATA_MANAGER_ANNOTATIONS_MAP.update(obj)
+
+
 class PreparedTaskManager(models.Manager):
     def get_queryset(self, fields_for_evaluation=None):
         queryset = TaskQuerySet(self.model)
+        annotations_map = get_annotations_map()
 
         if fields_for_evaluation is None:
             fields_for_evaluation = []
@@ -275,3 +310,8 @@ class PreparedTaskManager(models.Manager):
 
         fields_for_annotation = get_fields_for_annotation(prepare_params)
         return self.get_queryset(fields_for_annotation).prepared(prepare_params=prepare_params)
+
+
+class TaskManager(models.Manager):
+    def for_user(self, user):
+        return self.filter(project__organization=user.active_organization)
