@@ -7,8 +7,7 @@ from django.db import transaction
 
 from data_manager.models import View, Filter, FilterGroup
 from tasks.models import Task
-from tasks.serializers import TaskWithAnnotationsAndLazyPredictionsSerializer
-from django.db.models import Avg
+from tasks.serializers import TaskSerializer, AnnotationSerializer, PredictionSerializer
 
 
 class FilterSerializer(serializers.ModelSerializer):
@@ -156,7 +155,10 @@ class ViewSerializer(serializers.ModelSerializer):
             return instance
 
 
-class TaskSerializer(TaskWithAnnotationsAndLazyPredictionsSerializer):
+class DataManagerTaskSerializer(TaskSerializer):
+    predictions = PredictionSerializer(many=True, default=[], read_only=True)
+    annotations = AnnotationSerializer(many=True, default=[], read_only=True)
+
     cancelled_annotations = serializers.SerializerMethodField()
     completed_at = serializers.SerializerMethodField()
     annotations_results = serializers.SerializerMethodField()
