@@ -4,7 +4,7 @@ import logging
 
 from rest_framework import generics
 from rest_framework.views import APIView
-from core.permissions import BaseRulesPermission
+from core.permissions import all_permissions
 from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
 from rest_framework.response import Response
 
@@ -12,18 +12,14 @@ from .s3.api import S3ImportStorageListAPI, S3ExportStorageListAPI
 from .gcs.api import GCSImportStorageListAPI, GCSExportStorageListAPI
 from .azure_blob.api import AzureBlobImportStorageListAPI, AzureBlobExportStorageListAPI
 from .redis.api import RedisImportStorageListAPI, RedisExportStorageListAPI
-from .localfiles.api import LocalFilesImportStorageListAPI
+from .localfiles.api import LocalFilesImportStorageListAPI, LocalFilesExportStorageListAPI
 
 logger = logging.getLogger(__name__)
 # TODO: replace hardcoded apps lists with search over included storage apps
 
 
-class StorageAPIBasePermission(BaseRulesPermission):
-    perm = 'projects.change_project'
-
-
 class AllImportStorageTypesAPI(APIView):
-    permission_classes = (StorageAPIBasePermission,)
+    permission_required = all_permissions.projects_change
     swagger_schema = None
 
     def get(self, request, **kwargs):
@@ -37,7 +33,7 @@ class AllImportStorageTypesAPI(APIView):
 
 
 class AllExportStorageTypesAPI(APIView):
-    permission_classes = (StorageAPIBasePermission,)
+    permission_required = all_permissions.projects_change
     swagger_schema = None
 
     def get(self, request, **kwargs):
@@ -46,13 +42,14 @@ class AllExportStorageTypesAPI(APIView):
             {'name': 'gcs', 'title': 'Google Cloud Storage'},
             {'name': 'azure', 'title': 'Microsoft Azure'},
             {'name': 'redis', 'title': 'Redis'},
+            {'name': 'localfiles', 'title': 'Local files'},
         ])
 
 
 class AllImportStorageListAPI(generics.ListAPIView):
 
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (StorageAPIBasePermission,)
+    permission_required = all_permissions.projects_change
     swagger_schema = None
 
     def _get_response(self, api, request, *args, **kwargs):
@@ -80,7 +77,7 @@ class AllImportStorageListAPI(generics.ListAPIView):
 class AllExportStorageListAPI(generics.ListAPIView):
 
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (StorageAPIBasePermission,)
+    permission_required = all_permissions.projects_change
     swagger_schema = None
 
     def _get_response(self, api, request, *args, **kwargs):
@@ -93,5 +90,6 @@ class AllExportStorageListAPI(generics.ListAPIView):
             self._get_response(S3ExportStorageListAPI, request, *args, **kwargs),
             self._get_response(GCSExportStorageListAPI, request, *args, **kwargs),
             self._get_response(AzureBlobExportStorageListAPI, request, *args, **kwargs),
-            self._get_response(RedisExportStorageListAPI, request, *args, **kwargs)
+            self._get_response(RedisExportStorageListAPI, request, *args, **kwargs),
+            self._get_response(LocalFilesExportStorageListAPI, request, *args, **kwargs)
         ], []))
