@@ -11,8 +11,9 @@ from rest_framework.response import Response
 from rest_framework.exceptions import NotFound
 from drf_yasg import openapi as openapi
 from drf_yasg.utils import swagger_auto_schema
+from django.utils.decorators import method_decorator
 
-from core.permissions import BaseRulesPermission, IsBusiness, get_object_with_permissions
+from core.permissions import BaseRulesPermission, IsBusiness, get_object_with_permissions, all_permissions
 from core.utils.common import get_object_with_check_and_log
 from core.utils.io import read_yaml
 from io_storages.serializers import ImportStorageSerializer, ExportStorageSerializer
@@ -27,7 +28,7 @@ class StorageAPIBasePermission(BaseRulesPermission):
 
 class ImportStorageListAPI(generics.ListCreateAPIView):
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (IsBusiness, StorageAPIBasePermission)
+    permission_required = all_permissions.projects_change
     serializer_class = ImportStorageSerializer
 
     def get_queryset(self):
@@ -37,38 +38,12 @@ class ImportStorageListAPI(generics.ListCreateAPIView):
         ImportStorageClass = self.serializer_class.Meta.model
         return ImportStorageClass.objects.filter(project_id=project.id)
 
-    @swagger_auto_schema(tags=['Storage'], operation_summary='Get import storage', 
-                         operation_description='Retrieve storage details for configured source storage. '
-                                               'Use the relevant endpoint for the type of storage details you want to '
-                                               'retrieve.')
-    def get(self, request, *args, **kwargs):
-        return super(ImportStorageListAPI, self).get(request, *args, **kwargs)
-
-    @swagger_auto_schema(tags=['Storage'], operation_summary='Create import storage', 
-                         operation_description='Create a cloud or database storage connection to use as a source for '
-                                               'labeling tasks. Use the relevant endpoint for the type of storage you '
-                                               'want to create.')
-    def post(self, request, *args, **kwargs):
-        return super(ImportStorageListAPI, self).post(request, *args, **kwargs)
-
 
 class ImportStorageDetailAPI(generics.RetrieveUpdateDestroyAPIView):
     """RUD storage by pk specified in URL"""
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     serializer_class = ImportStorageSerializer
-    permission_classes = (IsBusiness, StorageAPIBasePermission)
-
-    @swagger_auto_schema(tags=['Storage'])
-    def get(self, request, *args, **kwargs):
-        return super(ImportStorageDetailAPI, self).get(request, *args, **kwargs)
-
-    @swagger_auto_schema(tags=['Storage'])
-    def patch(self, request, *args, **kwargs):
-        return super(ImportStorageDetailAPI, self).patch(request, *args, **kwargs)
-
-    @swagger_auto_schema(tags=['Storage'])
-    def delete(self, request, *args, **kwargs):
-        return super(ImportStorageDetailAPI, self).delete(request, *args, **kwargs)
+    permission_required = all_permissions.projects_change
 
     @swagger_auto_schema(auto_schema=None)
     def put(self, request, *args, **kwargs):
@@ -77,7 +52,7 @@ class ImportStorageDetailAPI(generics.RetrieveUpdateDestroyAPIView):
 
 class ExportStorageListAPI(generics.ListCreateAPIView):
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (IsBusiness, StorageAPIBasePermission)
+    permission_required = all_permissions.projects_change
     serializer_class = ExportStorageSerializer
 
     def get_queryset(self):
@@ -87,53 +62,30 @@ class ExportStorageListAPI(generics.ListCreateAPIView):
         ImportStorageClass = self.serializer_class.Meta.model
         return ImportStorageClass.objects.filter(project_id=project.id)
 
-    @swagger_auto_schema(tags=['Storage'], operation_summary='Get export storage', 
-                         operation_description='Retrieve storage details for configured target storage. Use the '
-                                               'relevant endpoint for the type of storage you want to retrieve.')
-    def get(self, request, *args, **kwargs):
-        return super(ExportStorageListAPI, self).get(request, *args, **kwargs)
-
-    @swagger_auto_schema(tags=['Storage'], operation_summary='Create export storage',
-                         operation_description='Create a cloud connection to store annotations. Use the relevant '
-                                               'endpoint for the type of storage you want to create.')
-    def post(self, request, *args, **kwargs):
-        return super(ExportStorageListAPI, self).post(request, *args, **kwargs)
-
 
 class ExportStorageDetailAPI(generics.RetrieveUpdateDestroyAPIView):
     """RUD storage by pk specified in URL"""
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     serializer_class = ExportStorageSerializer
-    permission_classes = (IsBusiness, StorageAPIBasePermission)
-
-    @swagger_auto_schema(tags=['Storage'])
-    def get(self, request, *args, **kwargs):
-        return super(ExportStorageDetailAPI, self).get(request, *args, **kwargs)
-
-    @swagger_auto_schema(tags=['Storage'])
-    def patch(self, request, *args, **kwargs):
-        return super(ExportStorageDetailAPI, self).patch(request, *args, **kwargs)
-
-    @swagger_auto_schema(tags=['Storage'])
-    def delete(self, request, *args, **kwargs):
-        return super(ExportStorageDetailAPI, self).delete(request, *args, **kwargs)
+    permission_required = all_permissions.projects_change
 
     @swagger_auto_schema(auto_schema=None)
     def put(self, request, *args, **kwargs):
         return super(ExportStorageDetailAPI, self).put(request, *args, **kwargs)
 
 
-class ImportStorageSyncAPI(APIView):
+class ImportStorageSyncAPI(generics.GenericAPIView):
 
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (IsBusiness, StorageAPIBasePermission)
+    permission_required = all_permissions.projects_change
     serializer_class = ImportStorageSerializer
 
-    @swagger_auto_schema(tags=['Storage'])
-    def post(self, request, *args, **kwargs):
+    def get_queryset(self):
         ImportStorageClass = self.serializer_class.Meta.model
-        storage = get_object_with_permissions(
-            request, ImportStorageClass, self.kwargs['pk'], StorageAPIBasePermission.perm)
+        return ImportStorageClass.objects.all()
+
+    def post(self, request, *args, **kwargs):
+        storage = self.get_object()
         # check connectivity & access, raise an exception if not satisfied
         storage.validate_connection()
         storage.sync()
@@ -152,10 +104,6 @@ class StorageValidateAPI(generics.CreateAPIView):
         storage.validate_connection()
         return Response()
 
-    @swagger_auto_schema(tags=['Storage'])
-    def post(self, request, *args, **kwargs):
-        return super(StorageValidateAPI, self).post(request, *args, **kwargs)
-
 
 class StorageFormLayoutAPI(generics.RetrieveAPIView):
 
@@ -164,6 +112,7 @@ class StorageFormLayoutAPI(generics.RetrieveAPIView):
     swagger_schema = None
     storage_type = None
 
+    @swagger_auto_schema(auto_schema=None)
     def get(self, request, *args, **kwargs):
         form_layout_file = os.path.join(os.path.dirname(inspect.getfile(self.__class__)), 'form_layout.yml')
         if not os.path.exists(form_layout_file):
