@@ -139,7 +139,7 @@ class AzureBlobExportStorage(ExportStorage, AzureBlobStorageMixin):
     def save_annotation(self, annotation):
         container = self.get_container()
         logger.debug(f'Creating new object on {self.__class__.__name__} Storage {self} for annotation {annotation}')
-        ser_annotation = StorageAnnotationSerializer(annotation).data
+        ser_annotation = self._get_serialized_data(annotation)
         with transaction.atomic():
             # Create export storage link
             link = AzureBlobExportStorageLink.create(annotation, self)
