@@ -147,7 +147,7 @@ class GCSImportStorage(ImportStorage, GCSStorageMixin):
         expires_at_ms = datetime.now() + timedelta(minutes=self.presign_ttl)
         # This next line is the trick!
         signing_credentials = compute_engine.IDTokenCredentials(auth_request, "",
-                                                                service_account_email=credentials.service_account_email)
+                                                                service_account_email=None)
         signed_url = signed_blob_path.generate_signed_url(expires_at_ms, credentials=signing_credentials, version="v4")
         return signed_url
 
@@ -168,7 +168,7 @@ class GCSExportStorage(ExportStorage, GCSStorageMixin):
     def save_annotation(self, annotation):
         bucket = self.get_bucket()
         logger.debug(f'Creating new object on {self.__class__.__name__} Storage {self} for annotation {annotation}')
-        ser_annotation = StorageAnnotationSerializer(annotation).data
+        ser_annotation = self._get_serialized_data(annotation)
         with transaction.atomic():
             # Create export storage link
             link = GCSExportStorageLink.create(annotation, self)
