@@ -10,17 +10,18 @@ class MLBackendSerializer(serializers.ModelSerializer):
         attrs = super(MLBackendSerializer, self).validate(attrs)
         url = attrs['url']
         if MLBackend.healthcheck_(url).is_error:
-            raise serializers.ValidationError(f'Can\'t connect to ML backend {url}. Is it valid? '
-                                              f'<a href="https://labelstud.io/guide/ml.html>Read more</a>'
-                                              f' how to set up your code as a ML backend')
+            raise serializers.ValidationError("Can't connect to ML backend {url}, health check failed. "
+                                              f'Make sure it is up and your firewall is properly configured. '
+                                              f'<a href="https://labelstud.io/guide/ml.html>Learn more</a>'
+                                              f' about how to set up an ML backend.')
         project = attrs['project']
         setup_response = MLBackend.setup_(url, project)
         if setup_response.is_error:
             raise serializers.ValidationError(
-                f'Successfully connected to {url} but it doesn\'t look like a valid ML backend. '
+                f"Successfully connected to {url} but it doesn't look like a valid ML backend. "
                 f'Reason: {setup_response.error_message}.\n'
-                f'Perhaps it\'s better to check ML backend server console logs to see what\'s going on there\n'
-                f'(may be something wrong with your model or it is incompatible with current labeling configuration?)')
+                f'Check the ML backend server console logs to check the status. There might be\n'
+                f'something wrong with your model or it might be incompatible with the current labeling configuration.')
         return attrs
 
     class Meta:
