@@ -39,8 +39,10 @@ class OrganizationMember(models.Model):
     def is_owner(self):
         return self.user.id == self.organization.created_by.id
 
+OrganizationMixin = load_func(settings.ORGANIZATION_MIXIN)
 
-class Organization(models.Model):
+
+class Organization(OrganizationMixin, models.Model):
     """
     """
     title = models.CharField(_('organization title'), max_length=1000, null=False)
@@ -89,6 +91,11 @@ class Organization(models.Model):
     def has_project_member(self, user):
         return self.projects.filter(members__user=user).exists()
 
+    def has_permission(self, user):
+        if self in user.organizations.all():
+            return True
+        return False
+
     def add_user(self, user):
         if self.users.filter(pk=user.pk).exists():
             logger.debug('User already exists in organization.')
