@@ -2,7 +2,6 @@
 """
 import logging
 
-from django.db import transaction
 from django.db.models import Q
 from django.utils import timezone
 
@@ -11,15 +10,15 @@ from drf_yasg.utils import swagger_auto_schema
 
 from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
 from rest_framework import generics
-from rest_framework import status
-from rest_framework.views import APIView
 from rest_framework.response import Response
+from rest_framework.permissions import IsAuthenticated
 
 from core.utils.common import get_object_with_check_and_log
+from core.decorators import permission_required
+from core.permissions import all_permissions, ViewClassPermission
 
 from tasks.models import Task, Annotation, Prediction, AnnotationDraft
-from core.permissions import (IsAuthenticated, IsBusiness, BaseRulesPermission,
-                              get_object_with_permissions, check_object_permissions)
+from core.permissions import (get_object_with_permissions, check_object_permissions)
 from core.mixins import RequestDebugLogMixin
 from core.utils.common import bool_from_request
 from tasks.serializers import (
@@ -30,14 +29,6 @@ from projects.models import Project
 logger = logging.getLogger(__name__)
 
 
-class TaskAPIViewTaskPermission(BaseRulesPermission):
-    perm = 'tasks.view_task'
-
-
-class TaskAPIChangeProjectPermission(BaseRulesPermission):
-    perm = 'projects.change_project'
-
-
 class TaskListAPI(generics.ListCreateAPIView):
     """
     post:
@@ -46,7 +37,6 @@ class TaskListAPI(generics.ListCreateAPIView):
     Create a new labeling task in Label Studio.
     """
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (IsAuthenticated, )
     queryset = Task.objects.all()
     serializer_class = TaskSerializer
 
@@ -58,8 +48,7 @@ class TaskListAPI(generics.ListCreateAPIView):
         context = super(TaskListAPI, self).get_serializer_context()
         project_id = self.request.data.get('project')
         if project_id:
-            context['project'] = get_object_with_permissions(
-                self.request, Project, project_id, TaskAPIChangeProjectPermission.perm)
+            context['project'] = generics.get_object_or_404(Project, pk=project_id)
         return context
 
     @swagger_auto_schema(tags=['Tasks'], request_body=TaskSerializer)
@@ -85,7 +74,6 @@ class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
     Delete a task in Label Studio. This action cannot be undone!
     """
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (IsAuthenticated, )
     queryset = Task.objects.all()
 
     def get_serializer_class(self):
@@ -97,18 +85,11 @@ class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
         else:
             return TaskSimpleSerializer
     
-    def get_object(self):
-        if self.request.method == 'GET':
-            obj = get_object_with_permissions(self.request, Task, self.kwargs['pk'], 'tasks.view_task')
-        else:
-            obj = get_object_with_permissions(self.request, Task, self.kwargs['pk'], 'projects.view_project')
-        return obj
-    
     def retrieve(self, request, *args, **kwargs):
         task = self.get_object()
 
         # call machine learning api and format response
-        if task.project.show_collab_predictions:
+        if task.project.evaluate_predictions_automatically:
             for ml_backend in task.project.ml_backends.all():
                 ml_backend.predict_one_task(task)
 
@@ -139,38 +120,6 @@ class TaskAPI(generics.RetrieveUpdateDestroyAPIView):
         return super(TaskAPI, self).put(request, *args, **kwargs)
 
 
-class TaskCancelAPI(APIView):
-    """
-    post:
-    Cancel Task
-
-    Set a labeling task as cancelled or skipped. 
-    """
-    swagger_schema = None
-    parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (IsAuthenticated,)
-    serializer_class = TaskSerializer
-
-    def post(self, request, *args, **kwargs):
-        # get the cancelled task
-        task = get_object_with_permissions(self.request, Task, self.kwargs['pk'], 'tasks.change_task')
-
-        # validate data from annotation
-        annotation = AnnotationSerializer(data=request.data)
-        annotation.is_valid(raise_exception=True)
-
-        # set annotator last activity
-        user = request.user
-        user.activity_at = timezone.now()
-        user.save()
-
-        # serialize annotation, update task and save
-        com = annotation.save(completed_by=user, was_cancelled=True, task=task)
-        task.annotations.add(com)
-        task.save()
-        return Response(annotation.data, status=status.HTTP_200_OK)
-
-
 class AnnotationAPI(RequestDebugLogMixin, generics.RetrieveUpdateDestroyAPIView):
     """
     get:
@@ -189,25 +138,25 @@ class AnnotationAPI(RequestDebugLogMixin, generics.RetrieveUpdateDestroyAPIView)
     Delete an annotation. This action can't be undone! 
     """
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    permission_classes = (IsAuthenticated, )
-    serializer_class = AnnotationSerializer
+    permission_required = ViewClassPermission(
+        GET=all_permissions.annotations_view,
+        PUT=all_permissions.annotations_change,
+        PATCH=all_permissions.annotations_change,
+        DELETE=all_permissions.annotations_delete,
+    )
 
-    def get_object(self):
-        return get_object_with_permissions(
-            self.request, Annotation, self.kwargs['pk'], 'annotations.view_annotation')
+    serializer_class = AnnotationSerializer
+    queryset = Annotation.objects.all()
 
     def perform_destroy(self, annotation):
-        check_object_permissions(self.request, annotation, 'annotations.delete_annotation')
         annotation.delete()
 
     def update(self, request, *args, **kwargs):
         # save user history with annotator_id, time & annotation result
-        update_id = self.request.user.id
         annotation_id = self.kwargs['pk']
         annotation = get_object_with_check_and_log(request, Annotation, pk=annotation_id)
-        task = annotation.task
 
-        task.save()  # refresh task metrics
+        annotation.task.save()  # refresh task metrics
 
         return super(AnnotationAPI, self).update(request, *args, **kwargs)
 
@@ -241,9 +190,11 @@ class AnnotationsListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
     Add annotations to a task like an annotator does.
     """
     parser_classes = (JSONParser, FormParser, MultiPartParser)
-    # Be careful: order of Permission Classes is important! It's lazy call of them.
-    # If HasAnnotatorProjectAccess is True, no more calls will be done. No raise exception about business.is_approved.
-    permission_classes = (IsBusiness, )
+    permission_required = ViewClassPermission(
+        GET=all_permissions.annotations_view,
+        POST=all_permissions.annotations_create,
+    )
+
     serializer_class = AnnotationSerializer
 
     @swagger_auto_schema(tags=['Annotations'])
@@ -255,13 +206,12 @@ class AnnotationsListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
         return super(AnnotationsListAPI, self).post(request, *args, **kwargs)
 
     def get_queryset(self):
-        task = get_object_with_permissions(self.request, Task, self.kwargs.get('pk', 0), 'tasks.view_task')
+        task = generics.get_object_or_404(Task.objects.for_user(self.request.user), pk=self.kwargs.get('pk', 0))
         return Annotation.objects.filter(Q(task=task) & Q(was_cancelled=False)).order_by('pk')
 
     def perform_create(self, ser):
         task = get_object_with_check_and_log(self.request, Task, pk=self.kwargs['pk'])
         # annotator has write access only to annotations and it can't be checked it after serializer.save()
-        check_object_permissions(self.request, Annotation(task=task), 'annotations.change_annotation')
         user = self.request.user
         # Release task if it has been taken at work (it should be taken by the same user, or it makes sentry error
         logger.debug(f'User={user} releases task={task}')
@@ -312,8 +262,11 @@ class AnnotationsListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
 class AnnotationDraftListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
 
     parser_classes = (JSONParser, MultiPartParser, FormParser)
-    permission_classes = (TaskAPIViewTaskPermission, )
     serializer_class = AnnotationDraftSerializer
+    permission_required = ViewClassPermission(
+        GET=all_permissions.annotations_view,
+        POST=all_permissions.annotations_create,
+    )
     queryset = AnnotationDraft.objects.all()
     swagger_schema = None
 
@@ -332,7 +285,12 @@ class AnnotationDraftListAPI(RequestDebugLogMixin, generics.ListCreateAPIView):
 class AnnotationDraftAPI(RequestDebugLogMixin, generics.RetrieveUpdateDestroyAPIView):
 
     parser_classes = (JSONParser, MultiPartParser, FormParser)
-    permission_classes = (TaskAPIViewTaskPermission, )
     serializer_class = AnnotationDraftSerializer
     queryset = AnnotationDraft.objects.all()
+    permission_required = ViewClassPermission(
+        GET=all_permissions.annotations_view,
+        PUT=all_permissions.annotations_change,
+        PATCH=all_permissions.annotations_change,
+        DELETE=all_permissions.annotations_delete,
+    )
     swagger_schema = None
