@@ -18,6 +18,7 @@ from ml.models import MLBackend
 from tasks.serializers import TaskWithAnnotationsSerializer
 from organizations.models import Organization
 from users.models import User
+
 try:
     from businesses.models import Business, BillingPlan
 except ImportError:
@@ -131,6 +132,18 @@ def azure_client_mock():
             yield
 
 
+@contextmanager
+def redis_client_mock():
+    from fakeredis import FakeRedis
+    from io_storages.redis.models import RedisStorageMixin
+
+    redis = FakeRedis()
+    # TODO: add mocked redis data
+
+    with mock.patch.object(RedisStorageMixin, 'get_redis_connection', return_value=redis):
+        yield
+
+
 def upload_data(client, project, tasks):
     tasks = TaskWithAnnotationsSerializer(tasks, many=True).data
     data = [{'data': task['data'], 'annotations': task['annotations']} for task in tasks]
