@@ -13,11 +13,11 @@ from rest_framework.authtoken.models import Token
 from rest_framework import viewsets
 from rest_framework.decorators import action
 from rest_framework import generics
+from rest_framework.permissions import IsAuthenticated
 
-from core.permissions import IsAuthenticated, CanModifyUserOrReadOnly
+from core.permissions import CanModifyUserOrReadOnly
 from users.models import User
 from users.serializers import UserSerializer
-from users.forms import UserProfileForm
 from users.functions import check_avatar
 
 
@@ -45,30 +45,56 @@ class UserAPI(viewsets.ModelViewSet):
             request.user.save()
             return Response(status=204)
 
-    @swagger_auto_schema(tags=['Users'], operation_summary='Save user details')
+    @swagger_auto_schema(
+        tags=['Users'],
+        operation_summary='Save user details',
+        operation_description='Save details for a specific user, such as their name, contact information, '
+                              'or organization in Label Studio.',
+        request_body=UserSerializer
+    )
     def update(self, request, *args, **kwargs):
-        form = UserProfileForm(data=request.data, files=request.FILES, instance=request.user)
-        if form.is_valid():
-            form.save()
-            return Response({'detail': 'user details saved'}, status=200)
+        return super(UserAPI, self).update(request, *args, **kwargs)
 
-    @swagger_auto_schema(auto_schema=None)
+    @swagger_auto_schema(
+        tags=['Users'],
+        operation_summary='List users',
+        operation_description='List the users that exist on the Label Studio server.'
+    )
     def list(self, request, *args, **kwargs):
         return super(UserAPI, self).list(request, *args, **kwargs)
 
-    @swagger_auto_schema(auto_schema=None)
+    @swagger_auto_schema(
+        tags=['Users'],
+        operation_summary='Create new user',
+        operation_description='Create a new user for Label Studio.',
+        request_body=UserSerializer
+    )
     def create(self, request, *args, **kwargs):
         return super(UserAPI, self).create(request, *args, **kwargs)
 
-    @swagger_auto_schema(auto_schema=None)
+    @swagger_auto_schema(
+        tags=['Users'],
+        operation_summary='Get user info',
+        operation_description='Get info about a specific Label Studio user.'
+    )
     def retrieve(self, request, *args, **kwargs):
         return super(UserAPI, self).retrieve(request, *args, **kwargs)
 
-    @swagger_auto_schema(auto_schema=None)
+    @swagger_auto_schema(
+        tags=['Users'],
+        operation_summary='Update user details',
+        operation_description='Update details for a specific user, such as their name, contact information, '
+                              'or organization in Label Studio.',
+        request_body=UserSerializer
+    )
     def partial_update(self, request, *args, **kwargs):
         return super(UserAPI, self).partial_update(request, *args, **kwargs)
 
-    @swagger_auto_schema(auto_schema=None)
+    @swagger_auto_schema(
+        tags=['Users'],
+        operation_summary='Delete user',
+        operation_description='Delete a specific Label Studio user.',
+    )
     def destroy(self, request, *args, **kwargs):
         return super(UserAPI, self).destroy(request, *args, **kwargs)
 
@@ -76,11 +102,12 @@ class UserAPI(viewsets.ModelViewSet):
 class UserResetTokenAPI(APIView):
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     queryset = User.objects.all()
-    permission_classes = (CanModifyUserOrReadOnly,)
+    permission_classes = (IsAuthenticated,)
 
     @swagger_auto_schema(
         tags=['Users'],
         operation_summary='Reset user token',
+        operation_description='Reset a specific user token.',
         responses={
             201: openapi.Response(
                 description='User token response',
@@ -109,6 +136,7 @@ class UserGetTokenAPI(APIView):
     @swagger_auto_schema(
         tags=['Users'],
         operation_summary='Get user token',
+        operation_description='Get a user token to authenticate to the API as a specific user.',
         responses={
             200: openapi.Response(
                 description='User token response',
@@ -123,12 +151,16 @@ class UserGetTokenAPI(APIView):
 class UserWhoAmIAPI(generics.RetrieveAPIView):
     parser_classes = (JSONParser, FormParser, MultiPartParser)
     queryset = User.objects.all()
-    permission_classes = (CanModifyUserOrReadOnly,)
+    permission_classes = (IsAuthenticated, )
     serializer_class = UserSerializer
 
     def get_object(self):
         return User.objects.get(id=self.request.user.id)
 
-    @swagger_auto_schema(tags=['Users'], operation_summary='Retrieve my user')
+    @swagger_auto_schema(
+        tags=['Users'],
+        operation_summary='Retrieve my user',
+        operation_description='Retrieve details the account that you are using to access the API.'
+    )
     def get(self, request, *args, **kwargs):
         return super(UserWhoAmIAPI, self).get(request, *args, **kwargs)
